﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Verse;

namespace TimerSwitches {
    [StaticConstructorOnStartup]
    public class TimeOfDaySwitch : Building_PowerSwitch {
        public const int HOURS_IN_DAY = GenDate.HoursPerDay;
        private static Texture2D icon;

        public static Texture2D OffColor => TimeAssignmentDefOf.Anything.ColorTexture;
        public static Texture2D OnColor => TimeAssignmentDefOf.Sleep.ColorTexture;
        
        private List<bool> states = new List<bool>();

        public override bool TransmitsPowerNow => GetState(GenDate.HourOfDay);

        public override void SpawnSetup() {
            base.SpawnSetup();

            for (int i = 0; i < HOURS_IN_DAY; i++) {
                states.Add(true);
            }

            LongEventHandler.ExecuteWhenFinished(() => icon = ContentFinder<Texture2D>.Get("UI/Commands/SetTime", true));
        }

        void designateTimes() {
            var allTabs = DefDatabase<MainTabDef>.AllDefs;

            var timersTab = allTabs.First(t => t.defName == "Timers");
            (timersTab.Window as TimersTab).CurrentTimer = this;
            Find.MainTabsRoot.SetCurrentTab(timersTab);
        }

        public override IEnumerable<Gizmo> GetGizmos() {
            List<Gizmo> baseGizmos = base.GetGizmos().ToList();
            for (int i = 0; i < baseGizmos.Count; i++) {
                Command baseGizmo = baseGizmos[i] as Command;
                // Suppress base power on/off switch
                if (baseGizmo != null && baseGizmo.defaultLabel == "CommandDesignateTogglePowerLabel".Translate()) continue;

                yield return baseGizmos[i];
            }

            yield return new Command_Action() {
                defaultLabel = "Set Times",
                defaultDesc = "Set the times that this switch is on or off.",
                hotKey = KeyBindingDefOf.Misc1,
                activateSound = SoundDef.Named("Click"),
                action = designateTimes,
                groupKey = 313123001,
                icon = icon,
            };
        }

        public void SetState(int hour, bool state) {
            states[hour] = state;
        }

        public bool GetState(int hour) { return states[hour]; }

        public override void Tick() {
            base.Tick();

            var hourOfDay = GenDate.HourOfDay;
            var state = GetState(hourOfDay);

            this.GetBaseInstanceField<CompFlickable>("flickableComp").SwitchIsOn = state;
            this.InvokeBaseMethod("UpdatePowerGrid");
        }

        public override void ExposeData() {
            base.ExposeData();

            Scribe_Collections.LookList(ref states, "timeOfDaySwitch");
        }
    }
}
